"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const matman_core_1 = require("matman-core");
/**
 * 页面操作类，使用这个类可以实现对浏览器页面的控制
 */
class PageDriverAsync {
    /**
     * 构造函数
     *
     * @param {Runner} browserRunner
     * @param {MatmanConfig} matmanConfig
     * @param {Object} [opts] 参数
     * @param {Object} [opts.delayBeforeRun] 延时多少ms再启动
     * @param {String} [opts.tag] 标记，在某些场景下使用，例如截图保存文件中追加该标记，用于做区分
     * @param {Boolean} [opts.useRecorder] 是否使用记录器
     * @param {Boolean} [opts.doNotCloseBrowser] 是否在执行完成之后不要关闭浏览器，默认为 false
     * @param {Boolean} [opts.nightmareConfig] 传递给 nightmare 的配置
     * @author helinjiang
     */
    constructor(browserRunner, matmanConfig, opts) {
        this.browserRunner = browserRunner;
        this.matmanConfig = matmanConfig;
        this.caseModuleFilePath = opts.caseModuleFilePath;
        this.show = !!opts.show;
        this.doNotCloseBrowser = !!opts.doNotCloseBrowser;
        this.useRecorder = !!opts.useRecorder;
        this.tag = opts.tag;
        // 特殊处理：如果 this.tag 是存在的文件，则获取文件名
        if (this.tag && fs_extra_1.default.existsSync(this.tag)) {
            this.tag = path_1.default.basename(this.tag).replace(/\./gi, '_');
        }
        // 延时多少ms再启动
        this.delayBeforeRun = typeof opts.delayBeforeRun === 'number' ? opts.delayBeforeRun : 0;
        // 设置浏览器
        this.executablePath = opts.executablePath;
        this.proxyServer = '';
        this.mockstarConfig = null;
        this.cookieConfig = null;
        this.deviceConfig = null;
        this.screenshotConfig = null;
        this.coverageConfig = null;
        this.matmanResultConfig = null;
        this.pageUrl = '';
        this.evaluateFn = null;
        this.evaluateFnArgs = [];
        this.actionList = [];
        this.dataIndexMap = {};
    }
    getRunner() {
        return this.browserRunner;
    }
    /**
     * 走指定的代理服务，由代理服务配置请求加载本地项目，从而达到同源测试的目的
     * 若不配置，则之前请求现网，亦可直接测试现网的服务
     *
     * https://github.com/segmentio/nightmare#switches
     *
     * @param {String} proxyServer 代理服务器，格式为 my_proxy_server.example.com:8080，例如 127.0.0.1:8899
     * @return {PageDriver}
     * @author helinjiang
     */
    async useProxyServer(proxyServer) {
        this.proxyServer = proxyServer;
        await Promise.resolve();
    }
    /**
     * 使用 mockstar 工具来做接口 mock 数据
     *
     * https://github.com/mockstarjs/mockstar
     *
     * @return {PageDriver}
     * @author helinjiang
     */
    async useMockstar(queryDataMap) {
        this.mockstarConfig = new matman_core_1.MockstarConfig({ queryDataMap });
        await Promise.resolve();
    }
    /**
     * 注入 cookie
     *
     * https://github.com/helinjiang/nightmare-handler/blob/master/docs/exCookies.md
     * https://github.com/helinjiang/nightmare-handler/tree/master/demo/extend-exCookies
     *
     * @param {CookieConfigOpts } cookieConfigOpts 支持 `mykey1=myvalue1; mykey2=myvalue2` 和 `{mykey1:'myvalue1', mykey2:'myvalue'}` 写法
     * @return {PageDriver}
     * @author helinjiang
     */
    async setCookieConfig(cookieConfigOpts) {
        this.cookieConfig = new matman_core_1.CookieConfig(cookieConfigOpts);
        await Promise.resolve();
    }
    /**
     * 设置无头浏览器设备参数
     *
     * https://github.com/helinjiang/nightmare-handler/blob/master/docs/exDevice.md
     *
     * {DeviceConfigOpts} deviceConfig 设备名或者设备配置
     * @return {PageDriver}
     * @author helinjiang
     */
    async setDeviceConfig(deviceConfig) {
        this.deviceConfig = new matman_core_1.DeviceConfig(deviceConfig);
        await Promise.resolve();
    }
    /**
     * 设置截屏参数，默认不截图
     *
     * @param {Boolean | String | Object} screenshotConfig
     * @return {PageDriver}
     * @author helinjiang
     */
    async setScreenshotConfig(screenshotConfig) {
        if (screenshotConfig) {
            this.screenshotConfig = new matman_core_1.ScreenshotConfig(this.matmanConfig, screenshotConfig, this.caseModuleFilePath, this.tag);
        }
        await Promise.resolve();
    }
    /**
     * 设置覆盖率参数
     *
     * @param {Boolean | String | Object} coverageConfig
     * @return {PageDriver}
     * @author helinjiang
     */
    async setCoverageConfig(coverageConfig) {
        if (coverageConfig) {
            this.coverageConfig = new matman_core_1.CoverageConfig(this.matmanConfig, coverageConfig, this.caseModuleFilePath, this.tag);
        }
        await Promise.resolve();
    }
    /**
     * 设置 MatmanResult 执行结果参数
     *
     * @param {Boolean | String | Object} matmanResultConfig
     * @return {PageDriver}
     * @author helinjiang
     */
    async setMatmanResultConfig(matmanResultConfig) {
        if (matmanResultConfig) {
            this.matmanResultConfig = new matman_core_1.MatmanResultConfig(this.matmanConfig, matmanResultConfig, this.caseModuleFilePath, this.tag);
        }
        await Promise.resolve();
    }
    /**
     * 加载指定的页面地址
     *
     * @param pageUrl
     * @return {PageDriver}
     * @author helinjiang
     */
    async setPageUrl(pageUrl) {
        this.pageUrl = pageUrl;
        await Promise.resolve();
    }
    /**
     * 增加测试动作
     *
     * @param {String} actionName 动作名称，后续可通过它来获得最后的数据
     * @param {Function} actionCall 执行函数，接受一个 nightmare 对象，可以直接操作
     * @return {PageDriver}
     * @author helinjiang
     */
    async addAction(actionName, actionCall) {
        if (typeof actionCall === 'function') {
            this.actionList.push(actionCall);
            this.dataIndexMap[actionName + ''] = this.actionList.length - 1;
        }
        else if (typeof actionName === 'function') {
            this.actionList.push(actionName);
        }
        else {
            throw new Error('addAction should assign function!');
        }
        await Promise.resolve();
    }
    async evaluate(fn, ...args) {
        this.evaluateFn = fn;
        this.evaluateFnArgs = args;
        // 直接返回结果
        return this.getResult();
    }
    /**
     * 结束，获取结果
     * @return {Promise<{}>}
     * @author helinjiang
     */
    async getResult() {
        var _a, _b;
        // 如果没有 actionList 则直接抛出错误
        if (!this.actionList.length) {
            throw new Error('No action! Please use addAction(name, callback) to add your actions!');
        }
        // 默认处理 coverage，根据 window.__coverage__
        if (!this.coverageConfig) {
            await this.setCoverageConfig(true);
        }
        // 默认处理 matmanResult
        if (!this.matmanResultConfig) {
            await this.setMatmanResultConfig(true);
        }
        // 默认处理 deviceConfig
        if (!this.deviceConfig) {
            await this.setDeviceConfig('Chrome');
        }
        (_a = this.browserRunner) === null || _a === void 0 ? void 0 : _a.setPageDriver(this);
        if (process.env.IS_IN_IDE) {
            return new Promise(resolve => {
                resolve(this);
            });
        }
        return (_b = this.browserRunner) === null || _b === void 0 ? void 0 : _b.getResult().then(matmanResult => {
            // 保存数据快照
            if (this.matmanResultConfig) {
                this.matmanResultConfig.save(matmanResult);
            }
            return matmanResult;
        });
    }
}
exports.default = PageDriverAsync;
